import { Component, ElementRef, OnInit, Renderer2, ViewEncapsulation } from '@angular/core';
import { DomSanitizer, Meta, Title } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { RouterListenerService } from '../../../services/routerListener.service';
import { Utils } from '../../../utils/utils';
import { Post } from '../../models/post.model';
import { Tag } from '../../models/tag.model';
import { PostService } from '../../services/post.service';
import { filterTags } from '../utils/NewsUtils';

@Component({
  selector: 'app-post-details',
  templateUrl: './post-details.component.html',
  styleUrls: ['./post-details.component.scss', '../../../../assets/ghost/cards.min.css'],
  encapsulation: ViewEncapsulation.None,
})
export class PostDetailsComponent implements OnInit {
  public post: Post;
  public filteredTags: Tag[] = [];

  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    private postService: PostService,
    private sanitizer: DomSanitizer,
    private routerListener: RouterListenerService,
    private readonly elementRef: ElementRef,
    private renderer: Renderer2,
    private title: Title,
    private meta: Meta,
  ) {}

  ngOnInit(): void {
    if (history.state.data) {
      // Case of post opened from the news page
      this.post = new Post(history.state.data);
      this.post.safeHtml = this.sanitizer.bypassSecurityTrustHtml(this.post.html);

      // Set page properties (title, og tags)
      this.setProperties();
    } else {
      // Case of post page opened directly in the browser
      const postSlug = this.activatedRoute.snapshot.paramMap.get('slug');
      this.postService.getPost(postSlug).subscribe((post) => {
        this.post = post.posts[0];
        this.post.safeHtml = this.sanitizer.bypassSecurityTrustHtml(this.post.html);

        // Set page properties (title, og tags)
        this.setProperties();
      });
    }
    // Add ghost JS
    const script = this.renderer.createElement('script');
    script.src = '/assets/ghost/cards.min.js';
    this.renderer.appendChild(this.elementRef.nativeElement, script);
  }

  private setProperties(): void {
    // Set meta tags (for linkedin and social networks publication)
    new Utils().setMetaTags(this.title, this.meta, this.router, this.post);

    this.getFilteredTags();
  }

  public backToPosts(): void {
    this.routerListener.goToPreviousUrl();
  }

  private getFilteredTags(): void {
    this.filteredTags = filterTags(this.post.tags || []);
  }
}
