import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { User } from '../../models/user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';
import { UserService } from './../../services/user.service';

@Component({
  selector: 'app-my-structures',
  templateUrl: './my-structures.component.html',
  styleUrls: ['./my-structures.component.scss'],
})
export class MyStructuresComponent implements OnInit {
  public userProfile: User;
  public structures: StructureWithOwners[] = [];
  public pendingStructures: StructureWithOwners[] = [];

  constructor(
    private profileService: ProfileService,
    private structureService: StructureService,
    private userService: UserService,
    private router: Router,
    public utils: Utils,
    private notificationService: NotificationService,
  ) {}

  ngOnInit(): void {
    this.profileService.getProfile().then((profile) => {
      this.userProfile = new User(profile);
      this.getStructuresFromProfile();
      this.getPendingStructuresFromProfile();
    });
  }

  private getStructuresFromProfile(): void {
    const fetchStructures = this.userProfile.structuresLink.map((structureId) =>
      this.structureService.getStructureWithOwners(structureId).toPromise(),
    );
    Promise.all(fetchStructures).then((structures) => {
      // Sort structures alphabetically
      this.structures = structures.sort((a, b) => a.structure.structureName.localeCompare(b.structure.structureName));
    });
  }

  private getPendingStructuresFromProfile(): void {
    const fetchPendingStructures = this.userProfile.pendingStructuresLink.map((pending) =>
      this.structureService.getStructureWithOwners(pending.id).toPromise(),
    );
    Promise.all(fetchPendingStructures).then((structures) => {
      // Sort pending structures alphabetically
      this.pendingStructures = structures.sort((a, b) =>
        a.structure.structureName.localeCompare(b.structure.structureName),
      );
    });
  }

  public goToStructuresManagement(): void {
    this.router.navigateByUrl('/profil/gestion-structures');
  }

  public goToFormStructure(): void {
    this.router.navigateByUrl('/formulaire/structure');
  }

  public cancelJoin(idStructure: string): void {
    this.userService.cancelJoin(idStructure, this.userProfile._id).subscribe({
      next: () => {
        const index = this.pendingStructures.map((s) => s.structure._id).indexOf(idStructure);
        this.pendingStructures.splice(index, 1);
        this.notificationService.showSuccess('La demande a été annulée avec succès');
      },
      error: (err) => {
        this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        console.error(err);
      },
    });
  }
}
